
import os
from spruce.Information import ID3
from spruce.Information import MP3Meta
from spruce import Utils

class Song:
	
	_file = ""
	_id3Tag = ""
	_bitrate = ""
	_filesize = ""
	_genre = ""
	_time = ""
	_id = ""
	parsedID3 = 0
	
	
	def __init__(self, file):
		if not os.path.exists(file):
			raise Exception("File not found: %s" % file) 
		
		self._file = file
		self.getBitrate()
		self.getTime()
		

	def isOnIPod(self):
		if(self.getFile().find("/mnt/ipod/") == 0):
			return 1
		else:
			return 0
		
		
	def _readID3Tags(self):
		if self._id3Tag == "":

			self._id3Tag = ID3.ID3(self._file)

 			if self._id3Tag.artist == "":
 				self._id3Tag.artist = self._parseArtist()
				self.parsedID3 = 1

			if self._id3Tag.album == "":
				self._id3Tag.album = self._parseAlbum()
				self.parsedID3 = 1
	
			if self._id3Tag.title == "":
				self._id3Tag.title = self._parseTitle()
				self.parsedID3 = 1
			
			try:
				self.genre = self._id3Tag.genres[self._id3Tag.genre]
			except:
				self.genre = "Unknown"
			
	def _readMetaInfo(self):
	
		if self._bitrate == "":
			info = MP3Meta.mp3info(self.getFile())
			self._bitrate = info["BITRATE"]
			self._time = (info["SS"] + (info["MM"] * 60)) * 1000

	

	def _parseArtist(self):
		filePathList = self.getFile().split("/")
		directory =  filePathList[-2]
		if directory.find("-") > 1:
			 artist = directory.split("-")[0]
			 artist = artist.replace("_", " ")
			 artist = artist.strip()
			 return artist
			 
		else:
			return directory


	def _parseAlbum(self):
		filePathList = self.getFile().split("/")
		directory =  filePathList[-2]
		if directory.find("-") > 1:
			 album = directory.split("-")[1]
			 album = album.replace("_", " ")
			 album = album.strip()
			 return album
			 
		else:
			return directory		


	def _parseTitle(self):
		filePathList = self.getFile().split("/")
		title = filePathList[-1].replace(".mp3", "")
		return title
	
		
		
	#
	# read/write property accessors
	#

	def setId(self, id):
		self._id = id

	def getId(self):
		return self._id

	def getFile(self):
		return self._file

	def setFile(self, file):
		self._file = file
		


	#
	# read property accessors
	#

	def getArtist(self):
 		self._readID3Tags()
 		return self._id3Tag.artist
		
	def getAlbum(self):
 		self._readID3Tags()
 		return self._id3Tag.album

		
	def getTitle(self):
 		self._readID3Tags()
 		return self._id3Tag.title
				
	def getGenre(self):
		self._readID3Tags()
		return self.genre

	def getTrackNumber(self):
		return 1

	def getAmountOfTracks(self):
		return 10

	def getYear(self):
		return 2002

	def getComment(self):
		self._readID3Tags()
		return self._id3Tag.title

	def getBitrate(self):
		self._readMetaInfo()
		return self._bitrate
		
	def getTime(self):
		self._readMetaInfo()
		return self._time
		

	def getFilesize(self):
		if self._filesize == "":
			return os.path.getsize(self.getFile())
		else:
			return self._filesize
		
		
	def getIpodCompatibleFile(self):
		path = self._file.replace("/mnt/ipod", "")
		path = path.replace("/", ":")
		
		return path

	def getCDNumber(self):
		return 0
	

	def getAmountOfCDs(self):
		return 0
		
		
		
	def equals(self, other):
		if self.getFilesize() != other.getFilesize():
			return 0
			
		if self.getArtist() != other.getArtist():
			return 0
		
		if self.getAlbum() != other.getAlbum():
			return 0
		
		
		if self.getTitle() != other.getTitle():
			return 0
		
		if self.getFile().split("/")[-1] != other.getFile().split("/")[-1]:
			return 0
		
		return 1
		

	def writeID3(self):
		Utils.debug(self, "writing id3 tag: %s"  % self.getIpodCompatibleFile())
		self._id3Tag.write(self.getFile())
