
from spruce.SongPlacer import SongPlacer
from spruce.SongContainers import MasterPlaylist
from spruce.SongContainers import Playlist

from spruce.Tasks import *
from spruce.iTunesDB import iTunesDBWriter
from spruce import Utils

import os
import sys



class iPod:
	
	SPRUCE_VERSION = "0.2.3"
	
	_DEBUG = 1
	
	_name = ""
	_songPlacer = ""
	_songContainer = ""
	_iTunesDBWriter = ""
	_playlists = list()
	_taskManager = ""
	_automaticDBWriting = 0

    # Create a class variable that will hold a reference
    # to the single instance of TestSingleton.
	
	instance = None

    # Define a helper class that will override the __call___
    # method in order to provide a factory method for iPod.

	class SingletonHelper:
		
		def __call__( self, *args, **kw ) :
		
            # If an instance of iPod does not exist,
            # create one and assign it to iPod.instance.
			
			if iPod.instance is None:
				object = iPod()
				iPod.instance = object

            # Return iPod.instance, which should contain
            # a reference to the only instance of iPod
            # in the system.
					
			return iPod.instance
				
    # Create a class level method that must be called to
    # get the single instance of iPod.	
	
	getInstance = SingletonHelper()
	
	
	def __init__(self):
		self._songPlacer = SongPlacer(self._getMointPoint())
		self._songContainer = MasterPlaylist()
		
		self._taskManager = TaskManager()
		self._taskManager._ipod = self
		
		
	def setName(self, name):
		self._name = name
		
		
	def	getName(self):
		return self._name
		
	
	def addSong(self, song):

		if song.isOnIPod() == 0 and self._songContainer.containsSong(song) == 0:
			task = Task(self._songPlacer.placeSong, song)
			self._taskManager.addTask(task)

		self._songContainer.addSong(song)
		
		
		return song



	def removeSong(self, song):
		stillExists = 0
		
		for playlist in self._playlists:
			if playlist.containsSong(song):
				stillExists = 1
		
		if stillExists == 0:
			self._songContainer.removeSong(song)
			#Utils.debug(self, "SONG REMOVING: really removing the song: %s" % song.getFile())
			if (song.getFile().find("/mnt/ipod/iPod_Control/Music/") >= 0):
				os.remove(song.getFile())
			else:
				raise Exception("tried to remove a file on the harddisk! This should never happen! Please report this bug!")
		else:
			pass
			#Utils.debug(self, "SONG REMOVING: just removing the reference")


	def setAutomaticDBWriting(self, value):
		self._automaticDBWriting = value
		self._taskManager.setAutomaticDBWriting(value)

	
	def _getMointPoint(self):
		return "/mnt/ipod"	

	
	def createPlaylist(self, name, playlistClass=Playlist):
		self._playlists.append(playlistClass(name))
		self._playlists[-1]._ipod = self
		
		return self._playlists[-1]

	def removePlaylist(self, toDeleteplaylist):
		i = 0
		
		for playlist in self._playlists:
			if playlist.getName() == toDeleteplaylist.getName():
				# have to also delete all the songs of the playlist
				
				songs = playlist.getSongs()
				del self._playlists[i]
				
				for song in songs:
					self.removeSong(song)
					
			i += 1
			

	def getPlaylists(self):
		return self._playlists

	def getMasterPlaylist(self):
		return self._songContainer


	def getXML(self):
		xml = ""
		for playlist in self._playlists:	
			xml += playlist.getXML()		


		return xml


		
		
	def write(self):
		iTunesDBWriter.getPlaylistsMethod = self.getPlaylists
		iTunesDBWriter.getMasterPlaylistMethod = self.getMasterPlaylist
		iTunesDBWriter.write()
		Utils.debug(self, "Wrote iTunesDB")



	def update(self):
		if self._automaticDBWriting == 1:
			self.write()

