import gobject
import gtk
from gtk import glade
from gtk import gdk
import urlparse
import urllib
import os

from GladeAccessor import GladeAccessor
from Playlist import Playlist
from spruce import iPod
from Preferences import Preferences
from FilesystemSongReader import FilesystemSongReader

from spruce import Utils



class PlaylistsManager:

	
	currentPlaylist = 0
	 
	_playlists = list()
	_model = ""
	_songViewer = ""
	_ipod = ""
	_preferences = ""
	
	COLUMN_ID = 0
	COLUMN_ICON = 1
	COLUMN_PLAYLIST_NAME = 2
	COLUMN_EDITABLE = 3
	COLUMN_OBJECT = 4
	
	_TARGET = [('text/uri-list', 0, 0)]
	
	
	PLAYLIST_ICON = "playlist-icon.png"
	

	def __init__(self):
		gladeAccessor = GladeAccessor.getInstance()
		self._ipod = iPod.getInstance()
		self._preferences = Preferences.getInstance()
		
		self.view = gladeAccessor.getWidget("playlistsView")
		self.view.connect ('cursor_changed', self._on_playlistChange)
		gladeAccessor.getWidget("addPlaylistButton").connect ('clicked', self._on_addList)
		gladeAccessor.getWidget("addPlaylistMenu").connect ('activate', self._on_addList)
		gladeAccessor.getWidget("removePlaylistButton").connect ('clicked', self._on_removeList)
		gladeAccessor.getWidget("removePlaylistMenu").connect ('activate', self._on_removeList)

		self.view.connect('drag_data_received', self._OnDragDragDataReceived)
		self.view.drag_dest_set(gtk.DEST_DEFAULT_ALL, self._TARGET, gtk.gdk.ACTION_COPY)			
		
		self._addColumns()
		self._createPlaylistModel()
	 	#self._createLibrary()
		self.view.set_model(self._model)
		

    	
    
	def setSongViewer(self, songViewer):
		self._songViewer = songViewer

	def addSongToCurrentPlaylist(self, song):
		self.getCurrentPlaylist().addSong(song)
		
	def addSongToPlaylistByNumber(self, playlistNumber, song):
		self._playlists[playlistNumber].addSong(song)
		
 	def getPlaylists(self):
		return self._playlists	

	def getCurrentPlaylist(self):
		return self._playlists[self.currentPlaylist]

	def getLastPlaylist(self):
		return self._playlists[len(self._playlists)-1]

	def getPlaylistByName(self, name):
		for playlist in self._playlists:
			if playlist.getName() == name:
				return playlist
		
		return None


	def createPlaylist(self, name, select = 1):
		self._playlists.append(Playlist(name))
		
		iter = self._model.append ()
		self._model.set (iter, self.COLUMN_ID, len(self._playlists)-1)
		self._model.set (iter, self.COLUMN_ICON, gdk.pixbuf_new_from_file(self._preferences.getDataDirectory() + "/" + self.PLAYLIST_ICON))
		self._model.set (iter, self.COLUMN_PLAYLIST_NAME, name)
		self._model.set (iter, self.COLUMN_EDITABLE, gtk.TRUE)
		self._model.set (iter, self.COLUMN_OBJECT, self._playlists[len(self._playlists)-1])

		
		if select != 0:
 			# selecting the just created playlist
			selection = self.view.get_selection()
			iter = self._model.get_iter(len(self._playlists) - 1)
			selection.select_iter(iter)

			column = self.view.get_column(2)
			path = self._model.get_path(iter)
			self.view.set_cursor(path, column, gtk.TRUE)
			self.view.grab_focus()
			self.currentPlaylist = len(self._playlists) - 1
			self._updatePlaylistView()

			# make the songview receiving drags
			if len(self._playlists) == 1:
				self._songViewer.enableDragging()
		
 
	def removeSelectedPlaylist(self):
		Utils.debug(self, "Removing Playlist")
		selection = self.view.get_selection()
		(model, iter) = selection.get_selected()

		if iter != None:
			playlistObject =  model.get_value(iter, self.COLUMN_OBJECT)
			path = self._model.get_path(iter)
			Utils.debug(self, "Current Path %s" % path)

			self._model.remove(iter)
			playlistIndex = self._playlists.index(playlistObject)
			del self._playlists[playlistIndex]
			self._ipod.removePlaylist(playlistObject)

			# select the next playlist
			# TODO! URGENT! gotta make that a bit more nice
			try:
				myPath = path[:-1] + (path[-1],)
				nextIter = self._model.get_iter(myPath)
				
				if myPath >= 0:
					selection = self.view.get_selection()
					selection.select_iter(nextIter)	
					self._updatePlaylistView()	
				
			except:
				myPath = path[:-1] + (path[-1] - 1,)

				if myPath[-1] >= 0:
					nextIter = self._model.get_iter(myPath)
					selection = self.view.get_selection()
					selection.select_iter(nextIter)			
					self._updatePlaylistView()
		
			# it was the last playlist, make the songview not receiving drags anymore
			# and empty the songview.
			if len(self._playlists) == 0:
				self._songViewer.disableDragging()
				emptyModel = gtk.ListStore(gobject.TYPE_UINT, 
								   gobject.TYPE_STRING, 
								   gobject.TYPE_STRING, 
								   gobject.TYPE_STRING,
								   gobject.TYPE_STRING, 
								   gobject.TYPE_PYOBJECT 
								  )	
				self._songViewer.view.set_model(emptyModel)
			
 			self._ipod.update()		
		

	#
	#	   THE PRIVAT METHODS
	#
        
	def _createPlaylistModel(self):
		self._model = gtk.ListStore(gobject.TYPE_UINT, gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_BOOLEAN, gobject.TYPE_PYOBJECT)

		for item in self._playlists:
			print "found one playlist"
			iter = self._model.append ()
			self._model.set (iter, self.COLUMN_ID, len(self._playlists)-1)
			self._model.set (iter, self.COLUMN_ICON, gdk.pixbuf_new_from_file(self._preferences.getDataDirectory() + "/" + self.PLAYLIST_ICON))
			self._model.set (iter, self.COLUMN_PLAYLIST_NAME, unicode (item.name, 'iso-8859-1'))
			self._model.set (iter, self.COLUMN_EDITABLE, gtk.TRUE)
			self._model.set (iter, self.COLUMN_OBJECT, item)
			
	def _addColumns(self):
		renderer = gtk.CellRendererPixbuf ()
		column = gtk.TreeViewColumn ("   ", renderer, pixbuf=self.COLUMN_ICON)

		self.view.append_column (column)
		renderer = gtk.CellRendererText ()
		renderer.connect('edited', self._on_cellEdited, (self._model, self.COLUMN_ID))
		column = gtk.TreeViewColumn ("  Playlist  ", renderer, text=self.COLUMN_PLAYLIST_NAME, editable=self.COLUMN_EDITABLE)
		self.view.append_column (column)   


	def _createLibrary(self):
		self._playlists.append(Library())
		iter = self._model.append ()
		self._model.set (iter, self.COLUMN_ID, len(self._playlists)-1)
		self._model.set (iter, self.COLUMN_ICON, gdk.pixbuf_new_from_file(self.PLAYLIST_ICON))
		self._model.set (iter, self.COLUMN_PLAYLIST_NAME, unicode ("Library", 'iso-8859-1'))
		self._model.set (iter, self.COLUMN_EDITABLE, gtk.FALSE)   	

        
	def _updatePlaylistView(self):
		# first checking out which kinda container is choosen at all
		selection = self.view.get_selection()
		(model, iter) = selection.get_selected()
		playlistObject =  model.get_value(iter, self.COLUMN_OBJECT)
		playlistId =  model.get_value(iter, self.COLUMN_ID)
		self._songViewer.view.set_model(self._playlists[self._playlists.index(playlistObject)].model)    	
		self.currentPlaylist = playlistId
			

	

	#
	#	   THE GUI CALLBACKS
	#
	def _on_cellEdited(self, cell, pathString, text, (model, column)):
	
		existingPlaylist = self.getPlaylistByName(text)
		
		if existingPlaylist != None and existingPlaylist != self.getCurrentPlaylist():
			# displays the dialog saying that a playlist
			# with this name already exists
			gladeAccessor = GladeAccessor.getInstance()
			window = gladeAccessor.getWidget("window")
		 	dialog = gtk.MessageDialog(window,
					 gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
		             gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,
		             "A playlist with the name \"%s\" already exists. Please use a different name." % text)
			dialog.run()
			dialog.destroy()
				
		elif existingPlaylist != None and existingPlaylist == self.getCurrentPlaylist():
			# the edited playlist is the current one and the name
			# was not really changed at all
			pass
				
		else:
		    # changes the name of the playlist
			iter = self._model.get_iter_from_string(pathString)
			path = self._model.get_path(iter)
			self._model.set(iter, self.COLUMN_PLAYLIST_NAME, text)
			# changing the name of the playlist object
			self._playlists[path[0]].name = text
			self._playlists[path[0]]._sprucePlaylistObject.setName(text)
			self._ipod.update()
	
		

	# creates a playlist for every dragged directory and adds the song
	# in the directory to the coresponding playlist.
	def _OnDragDragDataReceived(self, w, context, x, y, data, info, time):
		if data and data.format == 8:
			a = FilesystemSongReader()
			a.setSongdataHandler(self.addSongToCurrentPlaylist)			

			for uri in data.data.split('\r\n')[:-1]:
			
				path = urlparse.urlparse(urllib.unquote(uri))[2]
					
	 			if os.path.isdir(path):
					self.createPlaylist(path.split("/")[-1])
 					a.readDirectory(path)
 					

		context.finish(gtk.TRUE, gtk.FALSE, time)
		
		
		
		
	def _on_playlistChange(self, foo):
		self._updatePlaylistView()
		# make the songview receiving drags
		self._songViewer.enableDragging()
		
  
  	
	def _on_addList(self, button):
		self.createPlaylist(unicode ("New Playlist " + str(len(self._playlists)) , 'iso-8859-1'))
		self._updatePlaylistView()
       
	def _on_removeList(self, foo):
		self.removeSelectedPlaylist()  
		
