from __future__ import generators
import sys
import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade
import gnome
import gnome.ui
import os
import shutil
import md5

from xml.dom import Node, ext
from xml.dom.ext.reader import PyExpat


from Preferences import Preferences
from GladeAccessor import GladeAccessor
from SongViewer import SongViewer
from PlaylistsManager import PlaylistsManager

from spruce import iPod
from spruce.Songs import Song
from spruce.Songs import StaticSong
from spruce import Utils

class Whale:

	_ipod = ""
	_DEBUG = 0
	_IPOD_MOUNT_POINT = "/mnt/ipod/"
	
	APPNAME = "Whale"
	VERSION = "0.2.3"
	
	_warnBeforeSaving = 0
	
	
	def __init__(self):
		gnome.init(self.APPNAME, self.VERSION)
		
		# core setup of the library stuff
		self._ipod = iPod.getInstance()


		self.preferences = Preferences.getInstance()
		self.preferences.setup()		
 		self.gladeAccessor = GladeAccessor.getInstance()
		self.gladeAccessor.setFile(self.preferences.getDataDirectory() + "/whale.glade")

		# some sanity checks
		self._checkSpruceVersion()
		#self._checkIfIPodIsMounted()
		
		#self._ipod._songPlacer.deleteAllMusicFiles()
		self._ipod._taskManager.start()

		# setup of the whale components
		self.songViewer = SongViewer()
		self.playlistManager = PlaylistsManager()
		self.playlistManager.setSongViewer(self.songViewer)
		self.songViewer.setPlaylistManager(self.playlistManager)
		self.songViewer._ipod = self._ipod
		
		self._progressBar = self.gladeAccessor.getWidget("progressbar")
		self._statusbar = self.gladeAccessor.getWidget("statusbar")
		self.win = self.gladeAccessor.getWidget("window")
		self.gladeAccessor.getWidget("aboutMenu").connect('activate', self._on_aboutMenu_activate)
		self.gladeAccessor.getWidget("quitMenu").connect('activate', self._quit)
#		self.gladeAccessor.getWidget("preferencesMenu").connect('activate', self._on_preferences_activate)

		self.preferenceDialog = self.gladeAccessor.getWidget("preferencesDialog")			
			
		self._displayStatusBarMessage("Loading iPod Database")
		self._prepareDataLoading()
		
		self.win.show_all()
		self.win.connect('destroy', self._quit)
 		gtk.threads_init()
 		gtk.main()


	def _activateTaskManagerStatus(self):
		gtk.timeout_add(20, self._updateStatusMessages)


	def _prepareDataLoading(self):
		if not os.path.exists(self.preferences.getiPodPath() + "iPod_Control/iTunesDB.md5"):
 			self._warnBeforeSaving = 1
			self._showSavingPreWarning()
		else:
			iTunesDBmd5 = md5.new(open(self.preferences.getiPodPath() + 
				"iPod_Control/iTunes/iTunesDB").read()).hexdigest()
						
			if iTunesDBmd5 != open(self.preferences.getiPodPath() + "iPod_Control/iTunesDB.md5").read():
 				self._warnBeforeSaving = 1
				self._showSavingPreWarning()
								
		
		from spruce.iTunesDB import iTunesDBImport
		iTunesDBImport.createPlaylistMethod = self.playlistManager.createPlaylist
		iTunesDBImport.addSongMethod = self.playlistManager.addSongToPlaylistByNumber
		iTunesDBImport.finishedMethod = self._activateTaskManagerStatus
		gen = iTunesDBImport.importDatabase(self.preferences.getiPodPath() + "iPod_Control/iTunes/iTunesDB")
		gtk.idle_add(gen.next)	

	def _updateStatusMessages(self):
		if self._ipod._taskManager.getAmountOfOpenTasks() > 0:
			perc = self._ipod._taskManager.getPercentDone()
			self._displayStatusBarMessage(self._ipod._taskManager.getCurrentTaskDescription())
		else: 
			perc = 0
			self._displayStatusBarMessage("Done")
		
		# updating the progressbar
		self._progressBar.set_fraction(float(perc / 100))
		
		gtk.timeout_add(20, self._updateStatusMessages)
	

	def _displayStatusBarMessage(self, text):

		cid = self._statusbar.get_context_id("asdf")
		self._statusbar.pop(cid)
		self._statusbar.push(cid, " %s " % text)

		

	#
	# SANITY CHECKS
	#

	def _checkSpruceVersion(self):
		if self._ipod.SPRUCE_VERSION != self.VERSION:
			self._showErrorMessage("Spruce version does not match. Spruce %s is required" % self.VERSION) 
			sys.exit()
			
			
	def _checkIfIPodIsMounted(self):
		if not os.path.exists(self._IPOD_MOUNT_POINT + "iPod_Control"): 			
			self._showErrorMessage("The iPod is currently not mounted. Please mount it and restart Whale.")
			sys.exit()
			
			
	def _showErrorMessage(self, message):
		self.win.hide()
		dialog = gtk.MessageDialog(self.win,
						 gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
		            	 gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,
		            	 message)
		dialog.show()
		dialog.run()
		dialog.destroy()


	def _showSavingPostWarning(self):
		dialog = self.gladeAccessor.getWidget("SavingConfirmationDialog")
		dialog.show_all()
		self.gladeAccessor.getWidget("saveButton").connect('clicked', self._writeDatabaseAfterImport)
		dialog.run()
		dialog.destroy()


	def _writeDatabaseAfterImport(self, args):
		self._ipod.write()
		self._writeMD5File()
				

	def _showSavingPreWarning(self):
		dialog = gtk.MessageDialog(self.win,
						 gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
		            	 gtk.MESSAGE_INFO, gtk.BUTTONS_OK, 
						 "<span weight='bold' size='larger'>Loading your iPod Database\n\n</span>"
		            	 "It seems like its the first time you use Whale. Please make sure "
						 "that it's loaded correctly. When you quit Whale you will be asked "
						 "if you want to save the iPod database.")
		dialog.label.set_use_markup(gtk.TRUE)
		dialog.show()
		dialog.run()
		dialog.destroy()
	


	def _on_aboutMenu_activate(self, foo):
	 	gnome.ui.About(
            self.APPNAME, self.VERSION, "Copyright (c) 2003 Florian Weber", "",
            ["Florian Weber <florian@structbench.com>", 
             "iTunesDB writing code ported from Jorn Schulers itunesdb.c <Jorg.Schuler@gmx.de>\n"
			 "Whale logo by Marie-Chantale Turgeon <m-c@wheniridemybike.com>\n",""], [],
            "Florian Weber <florian@structbench.com>\n",
			gtk.gdk.pixbuf_new_from_file(self.preferences.getDataDirectory() + "/whale.png")).show()
 
 
 	def _on_preferences_activate(self, foo):
		self.preferenceDialog.show()


	def _writeMD5File(self):
		iTunesDBmd5 = md5.new(open(self.preferences.getiPodPath() + 
				"iPod_Control/iTunes/iTunesDB").read()).hexdigest()
		md5File = open(self.preferences.getiPodPath() + "iPod_Control/iTunesDB.md5", "w")
		md5File.write(iTunesDBmd5)
		
		
	def _quit(self, win):
		Utils.debug(self, "Quitting")
 		self._ipod._taskManager.stop()
	
		if self._warnBeforeSaving == 0:
			self._ipod.write()
			self._writeMD5File()
			
			gtk.main_quit()
		else:
			self._showSavingPostWarning()

			
		gtk.main_quit()
			
	


